local tradeskills = { 
	{ 	skillId = 171, -- Alchemy
		spellId = 2259,
		ranks = {
			{ 28677, {} }, -- Elixir master
			{ 28675, {} }, -- Potion master
			{ 28672, {} }, -- Transmutation master
			{ 51304, {} }, -- Grand Master
			{ 80731, {} }, -- Illustrious Grand Master
		}, },
	{ 	skillId = 164, -- Blacksmithing
		spellId = 2018,
		ranks = {
			{ 51300, {} }, -- Grand Master
			{ 76666, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 333, -- Enchanting
		spellId = 7411,
		ranks = {
			{ 51313, {} }, -- Grand Master
			{ 74258, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 202, -- Engineering
		spellId = 4036,
		ranks = {
			{ 20219, {} }, -- Gnomish Engineer
			{ 20222, {} }, -- Goblin Engineer
			{ 51306, {} }, -- Grand Master
			{ 82774, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 773, -- Inscription
		spellId = 45357,
		ranks = {
			{ 45363, {} }, -- Grand Master
			{ 86008, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 755, -- Jewelcrafting
		spellId = 25229,
		ranks = {
			{ 51311, {} }, -- Grand Master
			{ 73318, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 165, -- Leatherworking
		spellId = 2108,
		ranks = {
			{ 51302, {} }, -- Grand Master
			{ 81199, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 186, -- Mining
		spellId = 2575,
		ranks = {
			{ 50310, {} }, -- Grand Master
			{ 74517, {} }, -- Illustrious Grand Master
		}, },
	{	skillId = 197, -- Tailoring
		spellId = 3908,
		ranks = {
			{ 51309, {} }, -- Grand Master
			{ 75156, {} }, -- Illustrious Grand Master
		}, },
};

local _G = _G;
local next = _G.next;
local GameTooltip = _G.GameTooltip;
local CreateFrame = _G.CreateFrame;
local GetSpellInfo = _G.GetSpellInfo;
local GetNumGuildMembers = _G.GetNumGuildMembers;
local GetGuildRosterInfo = _G.GetGuildRosterInfo;
local ViewGuildRecipes = _G.ViewGuildRecipes;
local QueryGuildMembersForRecipe = _G.QueryGuildMembersForRecipe;
local GetGuildRecipeInfoPostQuery = _G.GetGuildRecipeInfoPostQuery;
local GetGuildRecipeMember = _G.GetGuildRecipeMember;

local parent;
local guildList = {};

local function GetOnlineColoredName(name)
	if guildList[name] then
		return "|cffffffff" .. name .. "|r";
	else
		return "|cff888888" .. name .. "|r";
	end
end

local function ShowTooltip(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(self.name);
	
	local skillLine = self:GetID();
	
	local ranks;
	for _, data in next, tradeskills do
		if data.skillId == skillLine then
			ranks = data.ranks;
			break;
		end
	end
	
	if ranks then
		for _, data in next, ranks do
			local spellId, playerNames = data[1], data[2];
			if #playerNames > 0 then
				local name, rank = GetSpellInfo(spellId);
				if rank and rank ~= "" then
					GameTooltip:AddLine(name .. " (" .. rank .. ")");
				else
					GameTooltip:AddLine(name);
				end
				local str = "";
				for _, name in next, playerNames do
					str = str .. GetOnlineColoredName(name) .. ", ";
				end
				GameTooltip:AddLine(str, 1, 0.82, 0, 1);
			end
		end
	end
	GameTooltip:Show();
end

local function UpdateOnlineList()
	local totalMembers, onlineMembers = GetNumGuildMembers();
	for i = 1, totalMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online = GetGuildRosterInfo(i);
		guildList[name] = online and true or false;
	end
end

local function SetUpButtons()
	parent = CreateFrame("Frame", "GuildProfessionsParentFrame", GuildFrame);
	parent:SetWidth(1);
	parent:SetHeight(1);
	parent:SetScale(0.85)
	parent:SetPoint("TOPLEFT", parent:GetParent(), "TOPRIGHT", 0, -36);
	parent:Show();

	local prev = nil;
	for _, data in next, tradeskills do
		local skillId, spellId = data.skillId, data.spellId;
		local button = CreateFrame("CheckButton", nil, parent, "SpellBookSkillLineTabTemplate");
		local name, _, icon = GetSpellInfo(spellId);
		button.name = name;
		
		button:SetID(skillId);
		button:SetNormalTexture(icon);
		button:Show();
		button:SetScript("OnEnter", ShowTooltip);
		button:SetScript("OnClick", function(this, button)
			ViewGuildRecipes(this:GetID());
			this:SetChecked(nil);
		end);

		if not prev then
			button:SetPoint("TOPLEFT", parent, "TOPRIGHT", 0, 0);
		else
			button:SetPoint("TOPLEFT", prev, "BOTTOMLEFT", 0, -17);
		end
		prev = button;
	end
end


local f = CreateFrame("Frame");
f:RegisterEvent("ADDON_LOADED");
f:RegisterEvent("GUILD_RECIPE_KNOWN_BY_MEMBERS");
f:RegisterEvent("GUILD_TRADESKILL_UPDATE");
f:RegisterEvent("GUILD_ROSTER_UPDATE");


local skillIndex = 1;
local rankIndex = 0;
local function ScanNextSpecialization()
	-- Clear timer, if any
	f.queryTimeout = 0;
	f:Hide();
	
	if skillIndex > #tradeskills then
		return
	end;
	local ranks = tradeskills[skillIndex].ranks;
	rankIndex = rankIndex + 1;
	if rankIndex > #ranks then
		skillIndex = skillIndex + 1;
		rankIndex = 0;
		return ScanNextSpecialization();
	end
	
	QueryGuildMembersForRecipe(tradeskills[skillIndex].skillId, ranks[rankIndex][1]);
	-- If no one in the guild has the skill, no event is fired
	-- so make a timeout to continue the scan, set it to a second
	f.queryTimeout = 1;
	f:Show();
end

local function ParseKnown()
	local skillLine, spellId, count = GetGuildRecipeInfoPostQuery();
	for _, skills in next, tradeskills do
		if skills.skillId == skillLine then
			for _, ranks in next, skills.ranks do
				if ranks[1] == spellId then
					local t = ranks[2];
					for i = 1, count do
						t[i] = GetGuildRecipeMember(i);
					end
			
					ScanNextSpecialization();
					return;
				end
			end
		end
	end
end

f:SetScript("OnEvent", function(this, event, ...)
	if event == "ADDON_LOADED" then
		local addonName = ...;
		if addonName == "Blizzard_GuildUI" then
			SetUpButtons();
			this:UnregisterEvent("ADDON_LOADED");
		end
	elseif event == "GUILD_RECIPE_KNOWN_BY_MEMBERS" then
		ParseKnown();
	elseif event == "GUILD_TRADESKILL_UPDATE" then
		f:UnregisterEvent("GUILD_TRADESKILL_UPDATE");
		ScanNextSpecialization();
	elseif event == "GUILD_ROSTER_UPDATE" then
		UpdateOnlineList();
	end
end);
f:SetScript("OnUpdate", function(self, elapsed)
	if self.queryTimeout then
		self.queryTimeout = self.queryTimeout - elapsed;
		if self.queryTimeout <= 0 then
			self:Hide();
			ScanNextSpecialization();
		end
	end
end);